#include <stm32f10x.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_gpio.h>
#include <stm32f10x_tim.h>
#include <misc.h>
#include "ds18b20.h"

// (. 1***)
/*      .
      GPIO_Mode_IN_FLOATING      GPIO_Mode_Out_OD,
             , 
    GPIO_Mode_IN_FLOATING,   
   GPIOA->BSRR = debug_pin; //     ODR
     .     .
*/

//       (Scratchpad  )
//   ""  
static uint8_t * dataBuf;

static uint8_t inProcess = 0;
static uint8_t typeOp = 0;

static uint8_t dataC = 0, dataB = 0; //  /
static uint8_t  nbit = 0; //  

//    ().
// :  typeOp = 4    .
static uint8_t  desBits = 0;

//  / (      )
static uint8_t currentCommandsPart;
static const uint8_t * currentCommandsPointer; //  / .
static uint8_t inputStateNow;

static uint16_t resultingValue;

static uint8_t mainResolution = resolution_12bit;
static uint16_t maxConversionTime = (uint16_t)999; // 830 ms

struct SearchRomStr{
	uint8_t lastDiscrepancy;
	uint8_t doneFlag;
	uint8_t romBit[8];
};

uint8_t tempCommand;

static uint8_t findNextROM(struct SearchRomStr * SearchRomStr1, uint8_t Command, uint8_t * userdataBuf);

//        (two's complement)
//          ,
//       ""   two's complement,
//            .
float ds18b20_convertTemp(const uint8_t * pScratchpadData){

	uint8_t negative;
	float temperature;
	uint16_t twoscomplementTemp;

	//  ; ssssssNNNNNNNnnnn
	twoscomplementTemp = ( (uint16_t)pScratchpadData[1] << 8 ) | (uint16_t)pScratchpadData[0];

	if ( (twoscomplementTemp & mask_sign ) == mask_sign ){
		negative = 1;

		twoscomplementTemp--; //    -   
		twoscomplementTemp = ~twoscomplementTemp;  //    (  )

	}else
		negative = 0;

	temperature = (float)(twoscomplementTemp >> 4); //   

	if( (twoscomplementTemp & mask_bit3) == mask_bit3  )
			temperature += 0.5 ; // 1*(1/2)

	if ( mainResolution >= resolution_10bit )
		if( (twoscomplementTemp & mask_bit2) == mask_bit2  )
			temperature += 0.25 ; // 1*(1/4)

	if ( mainResolution >= resolution_11bit )
		if( (twoscomplementTemp & mask_bit1) == mask_bit1  )
			temperature += 0.125 ; // 1*(1/8)

	if ( mainResolution == resolution_12bit )
		if( (twoscomplementTemp & mask_bit0) == mask_bit0  )
			temperature += 0.0625 ; // 1*(1/16)

	//     

	if ( negative )
		temperature *= -1;

	return temperature;
}


uint16_t ds18b20_initStep(void){

	GPIO_InitTypeDef	GPIO_InitStructure;

//    550 .  reset pulse.
	ds18b20_setTimerMode(0);

	GPIOA->BRR = pin_main_signal; // down  //    

	TIM1->CNT = 0;
	TIM1->ARR = 550;   //  480 

	inProcess = 1;
	NVIC_EnableIRQ(TIM1_UP_IRQn);

	TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS;  // ENABLE tim1
	typeOp = 1;

//GPIOA->BSRR =  GPIO_Pin_2;         // SET 
	while ( inProcess );
	typeOp = 0;
//GPIOA->BRR = GPIO_Pin_2;         // RESET 

	GPIOA->BSRR = pin_main_signal;  // UP
//  reset pulse.

	ds18b20_setTimerMode(1);

	inProcess = 1;
	TIM1->CNT = 0;

	NVIC_EnableIRQ(TIM1_UP_IRQn);
	NVIC_EnableIRQ(TIM1_CC_IRQn);

	typeOp = 5;
	TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS;   //TIM_Cmd(TIM1, ENABLE);

	while ( inProcess );
	typeOp = 0;

	return resultingValue;

}

void ds18b20_sendCommandData(const uint8_t * command, uint8_t desBitsPar){

	inProcess = 1;
	currentCommandsPart = *command; //command[0]
	currentCommandsPointer = command;

	//         
	if (inputStateNow)
		ds18b20_setTimerMode(0);

	if ( desBitsPar != 0 )
			desBits = desBitsPar;
	else
		desBits = 8;

	dataC = 0; //  
	dataB = 0; //  ()

	TIM1->CNT = 0;
	TIM1->ARR = 5;   //         . ?
	NVIC_EnableIRQ(TIM1_UP_IRQn);
	//NVIC_EnableIRQ(TIM1_CC_IRQn);

	typeOp = 2;
	//   URS  ,   TIM_FLAG_Update    
	TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS;     // ENABLE tim1

	while ( inProcess );
	typeOp = 0;
}

void ds18b20_getData(uint8_t desBitsPar, uint8_t * userdataBuf){
	uint8_t i;

	inProcess = 1;
	//         
	if (inputStateNow)
		ds18b20_setTimerMode(0);

	TIM1->CNT = 0;
	TIM1->ARR = 5;   // ???
	NVIC_EnableIRQ(TIM1_UP_IRQn);

	if ( desBitsPar != 0 ){
		desBits = desBitsPar;
		dataB = 0; //  
		dataC = 0; //     
		nbit = 0;  //  

		dataBuf = userdataBuf;
		// n    
		for( i = 0; i < (desBitsPar/8 + ( (desBitsPar % 8) > 0 ? 1 : 0)) ; i++)
			dataBuf[i] = 0;
		typeOp = 3;
	}else {
		desBits = 0;
		typeOp = 4;
	}

	//   URS  ,   TIM_FLAG_Update 
	//   
	TIM1->CR1 |= TIM_CR1_CEN  | TIM_CR1_URS;     // ENABLE tim1

	while ( inProcess );

	typeOp = 0;
}

void ds18b20_init(void){

	GPIO_InitTypeDef	GPIO_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	TIM_ICInitTypeDef TIM_ICInitStructure;

	//RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_TIM1, ENABLE);

	NVIC_InitStructure.NVIC_IRQChannel = TIM1_UP_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;
	NVIC_Init(&NVIC_InitStructure);

	NVIC_InitStructure.NVIC_IRQChannel = TIM1_CC_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;
	NVIC_Init(&NVIC_InitStructure);

	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure );
	TIM_TimeBaseStructure.TIM_Prescaler = SystemCoreClock /1000000 - 1; // "72" ( 1 mks per tick )
	TIM_TimeBaseStructure.TIM_Period = 20000 - 1; // 0..19999  (    60  240  )
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit (TIM1, &TIM_TimeBaseStructure );

	TIM_SelectInputTrigger(TIM1 , TIM_TS_TI1FP1);
	//TIM_SelectSlaveMode(TIM1 , TIM_SlaveMode_Reset); //    ds18b20_setTimerMode()
	TIM_SelectMasterSlaveMode(TIM1 , TIM_MasterSlaveMode_Enable);

}

void initDebug(void){
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed	= GPIO_Speed_2MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

void ds18b20_setTimerMode(uint8_t input) {
    GPIO_InitTypeDef	GPIO_InitStructure;

    TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
    TIM_ICInitTypeDef TIM_ICInitStructure;

    TIM_OCInitTypeDef TIM_OCInitStructure;

	inputStateNow = input;  //  ????

    if (input){

    	TIM1->ARR = 260; //    60  240 

		GPIO_StructInit(&GPIO_InitStructure);
		GPIO_InitStructure.GPIO_Pin = pin_main_signal; // trigger pin
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
		GPIO_InitStructure.GPIO_Speed	= GPIO_Speed_50MHz; //GPIO_Speed_2MHz;
		GPIO_Init(GPIOA, &GPIO_InitStructure);

		TIM_ICInitStructure.TIM_Channel = TIM_Channel_1;
		TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Falling;
		TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI; //  ,   1
		TIM_ICInitStructure.TIM_ICPrescaler = 0;
		TIM_ICInitStructure.TIM_ICFilter = 0;
		TIM_ICInit(TIM1, &TIM_ICInitStructure);

		TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;
		TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
		TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_IndirectTI; //  ,    1 ( 2 )
		TIM_ICInitStructure.TIM_ICPrescaler = 0;
		TIM_ICInitStructure.TIM_ICFilter = 0;
		TIM_ICInit(TIM1 , &TIM_ICInitStructure);

		// TIM_ITConfig(TIM1, TIM_IT_CC1 | TIM_IT_CC2 | TIM_IT_Update, ENABLE);
		TIM1->DIER |= TIM_IT_CC1 | TIM_IT_CC2 | TIM_IT_Update;

		/* Set the SMS Bits */
		//TIM_SelectSlaveMode(TIM1 , TIM_SlaveMode_Reset);
		TIM1->SMCR &= (uint16_t)~((uint16_t)TIM_SMCR_SMS); // reset
		TIM1->SMCR |= TIM_SlaveMode_Reset;

    } else { //  OUT

    	GPIOA->BSRR = pin_main_signal; // 1*** (. )
    	//GPIOA->ODR = GPIOA->ODR | pin_main_signal;

    	GPIO_StructInit(&GPIO_InitStructure);
		GPIO_InitStructure.GPIO_Pin = pin_main_signal;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;
		GPIO_InitStructure.GPIO_Speed	= GPIO_Speed_50MHz;
		GPIO_Init(GPIOA, &GPIO_InitStructure);

		TIM1->ARR = 1;

		TIM_OCStructInit(&TIM_OCInitStructure ); // << TIM_OCMode_Timing
		TIM_OC1Init(TIM1, &TIM_OCInitStructure );
		TIM_OC2Init(TIM1, &TIM_OCInitStructure );

		//TIM_ITConfig(TIM1, TIM_IT_CC1 | TIM_IT_CC2 , DISABLE);
		TIM1->DIER &= (uint16_t)~(TIM_IT_CC1 | TIM_IT_CC2);
		TIM1->DIER |= TIM_IT_Update;  // TIM_ITConfig(TIM1, TIM_IT_Update , ENABLE);

		/* Reset the SMS Bits */
		TIM1->SMCR &= (uint16_t)~((uint16_t)TIM_SMCR_SMS);
    }

}

void TIM1_UP_IRQHandler(void){

	static uint8_t cc = 0, log1 = 1;
	static uint8_t bit = 0;

	if ( TIM1->SR & TIM_FLAG_Update ){

		TIM1->SR = (uint16_t)~TIM_IT_Update; //TIM_ClearITPendingBit(TIM1, TIM_IT_Update);
		//asm("nop");
		//asm("nop");

		if ( (TIM1->CR1 & TIM_CR1_CEN) == 0 ) //    
			return;


		if ( typeOp == 1 ){      // .

			NVIC_DisableIRQ(TIM1_UP_IRQn);
			TIM_Cmd(TIM1, DISABLE);
			inProcess = 0;

			//TIM1->SR = (uint16_t)~TIM_IT_Update;
			return;
		}else if( typeOp == 2 ) {  //  /
				// out

					if ( dataC < desBits ) {

						if ( cc == 0 ){ //   - ""

							// GPIO_ResetBits(GPIOA, pin_main_signal);
							GPIOA->BRR = pin_main_signal;

							if (( currentCommandsPart & 0x1) == 1){

								log1 = 1;
								TIM1->ARR = 9; //TIM_Period = 9; // 10 

							}else {
								log1 = 0;
								TIM1->ARR = 69; //TIM_Period = 69; // 70 
							}

							currentCommandsPart = currentCommandsPart >> 1;
							cc++;
						}else {
							//GPIO_SetBits(GPIOA, pin_main_signal);
							GPIOA->BSRR = pin_main_signal;
							if ( log1 ){
								TIM1->ARR = 69; //TIM_Period = 69;
							} else { //log0
								TIM1->ARR = 9; //TIM_Period = 9;
							}

							dataC++;

							if ( (dataC % 8) == 0 )
								currentCommandsPart = currentCommandsPointer[++dataB];

							cc = 0;

						}

					} else {

							NVIC_DisableIRQ(TIM1_UP_IRQn);
							TIM_Cmd(TIM1, DISABLE);
							inProcess = 0;

					}
		} else if( typeOp == 3 ){
			//      (  )

			if( nbit < desBits ){
				if ( cc == 0 ){ //   - ""
					TIM1->ARR = 5; // period
					//TIM1->CNT = 0; //   ?
					GPIOA->BRR = pin_main_signal; // reset

					cc = 1;
				}else if ( cc == 1 ){

					//  15          
					TIM1->ARR = 8;
					GPIOA->BSRR = pin_main_signal; // set
					cc = 2;

				} else { // cc == 2

					TIM1->ARR = 69; // period
					// .    Input
					bit = (GPIOA->IDR & pin_main_signal ? 1 : 0);
					dataBuf[dataB] = dataBuf[dataB] | ( bit << dataC );

					nbit++;
					if ( (nbit % 8) == 0 ){
						dataB++;
						dataC = 0;
					}else
						dataC++;

					cc = 0;
				}
			} else {

				NVIC_DisableIRQ(TIM1_UP_IRQn);
				TIM_Cmd(TIM1, DISABLE);
				inProcess = 0;
			}
		}else if( typeOp == 4 ){
			//    (      1)

			if( desBits == 0){
				if ( cc == 0 ){ //   - ""
					TIM1->ARR = 5; // period
					//TIM1->CNT = 0; //   ?
					GPIOA->BRR = pin_main_signal; // reset

					cc = 1;
				}else if ( cc == 1 ){

					//  15          
					TIM1->ARR = 8;
					GPIOA->BSRR = pin_main_signal; // set
					cc = 2;

				} else { // cc == 2

					TIM1->ARR = 69; // period
					// bit = 0 :  
					// bit = 1 :  
					// .    Input
					desBits = (GPIOA->IDR & pin_main_signal ? 1 : 0);

					cc = 0;
				}
			} else {

				NVIC_DisableIRQ(TIM1_UP_IRQn);
				TIM_Cmd(TIM1, DISABLE);
				inProcess = 0;
			}
		}else if( typeOp == 5 ){
			//  ,      Reset pulse 
			//     Presence pulse.
			NVIC_DisableIRQ(TIM1_UP_IRQn);
			NVIC_DisableIRQ(TIM1_CC_IRQn);
			TIM1->CR1 &= (uint16_t)(~((uint16_t)TIM_CR1_CEN)); //TIM_Cmd(TIM1, DISABLE);
			resultingValue = 0;
			inProcess = 0;
		}
	}

}

void TIM1_CC_IRQHandler(void){

	static uint16_t vCCR;

	if (inputStateNow) {

		if ( (TIM1->CR1 & TIM_CR1_CEN) == 0 ){ //   
			TIM1->SR = (uint16_t)~(TIM_FLAG_CC1 | TIM_FLAG_CC2);
			return;
		}

		if ( TIM1->SR & TIM_FLAG_CC1 ){

			TIM1->SR = (uint16_t)~TIM_FLAG_CC1;

			(TIM1)->CNT = 0;
			vCCR = TIM1->CCR1;

		}else if ( TIM1->SR & TIM_FLAG_CC2 ){

			TIM1->SR = (uint16_t)~TIM_FLAG_CC2;

			NVIC_DisableIRQ(TIM1_UP_IRQn);
			NVIC_DisableIRQ(TIM1_CC_IRQn);
			TIM1->CR1 &= (uint16_t)(~((uint16_t)TIM_CR1_CEN)); //TIM_Cmd(TIM1, DISABLE);
			resultingValue = TIM1->CCR2;
			inProcess = 0;

		}
	}

}

uint8_t ds18b20_ConvertT(const uint8_t * ROMcode){

	uint16_t presencePulseTime;

	presencePulseTime = ds18b20_initStep();
	if (presencePulseTime < 60 || presencePulseTime > 240){
		return 1;	// err
	}

	//  65 . ,   .

		ds18b20_setTimerMode(0);
		TIM1->CNT = 0;
		TIM1->ARR = 65;

		inProcess = 1;
		NVIC_EnableIRQ(TIM1_UP_IRQn);
		TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS; // ENABLE tim1
		typeOp = 1;

		while ( inProcess );
		typeOp = 0;

	//  

	if ( ROMcode == 0 ){
		tempCommand = SkipRom;
		ds18b20_sendCommandData(&tempCommand, 0);
	}else{
		tempCommand = MatchRom;
		ds18b20_sendCommandData(&tempCommand, 0);
		ds18b20_sendCommandData(ROMcode, 64);
	}

	tempCommand = ConvertT;
	ds18b20_sendCommandData(&tempCommand, 0);

	//         

	//   103  830  .   .    
		ds18b20_setTimerMode(0);
		//TIM1->CNT = 0;
		TIM1->PSC = 60000-1; // Prescaler . =>> 1200    ( 830 )
		TIM1->EGR = TIM_PSCReloadMode_Immediate;

		TIM1->ARR = maxConversionTime;

		typeOp = 1;

		inProcess = 1;
		NVIC_EnableIRQ(TIM1_UP_IRQn);
		TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS;   //TIM_Cmd(TIM1, ENABLE);

		while ( inProcess );

		//   
		TIM1->PSC = SystemCoreClock /1000000 - 1; // "72"  ( 1    );
		TIM1->EGR = TIM_PSCReloadMode_Immediate;
		typeOp = 0;
	//  

	return 0;
}

uint8_t ds18b20_readScratchpad(const uint8_t * ROMcode, uint8_t * userdataBuf){

	uint16_t presencePulseTime;

	presencePulseTime = ds18b20_initStep();
	if (presencePulseTime < 60 || presencePulseTime > 240){
		return 1;	//err
	}

	//  65 
		ds18b20_setTimerMode(0);
		TIM1->CNT = 0;
		TIM1->ARR = 65;

		inProcess = 1;
		NVIC_EnableIRQ(TIM1_UP_IRQn);
		TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS; // ENABLE tim1;
		typeOp = 1;

		while ( inProcess );
		typeOp = 0;
	//  

	if ( ROMcode == 0 ){
		tempCommand = SkipRom;
		ds18b20_sendCommandData(&tempCommand, 0);
	}else{
		tempCommand = MatchRom;
		ds18b20_sendCommandData(&tempCommand, 0);
		ds18b20_sendCommandData(ROMcode, 64);
	}

	tempCommand = ReadScratchpad;
	ds18b20_sendCommandData(&tempCommand, 0);
	ds18b20_getData(72, userdataBuf);

	return 0;
}

uint8_t ds18b20_WriteScratchpad(const uint8_t * ROMcode, const int8_t * ThTlConfReg){
	uint16_t presencePulseTime;

	presencePulseTime = ds18b20_initStep();
	if (presencePulseTime < 60 || presencePulseTime > 240){
		return 1;	//err
	}

	//  65 
		ds18b20_setTimerMode(0);
		TIM1->CNT = 0;
		TIM1->ARR = 65;

		inProcess = 1;
		NVIC_EnableIRQ(TIM1_UP_IRQn);
		//NVIC_EnableIRQ(TIM1_CC_IRQn);
		TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS;     // ENABLE tim1;
		typeOp = 1;

		while ( inProcess );
		typeOp = 0;
	//  

	tempCommand = MatchRom;
	ds18b20_sendCommandData(&tempCommand, 0);
	ds18b20_sendCommandData(ROMcode, 64);

	tempCommand = WriteScratchpad;
	ds18b20_sendCommandData(&tempCommand, 0);
	ds18b20_sendCommandData(ThTlConfReg, 24); // int8_t - > uint8_t

	return 0;
}
//  Th, Tl - 
//  ConfReg   = 0,     
//         uint8_t 
//        int8_t.
uint8_t ds18b20_TestThTlConfReg(const uint8_t * ScratchpadData, const int8_t * ThTlConfReg){

	if ( (int8_t)ScratchpadData[2] == ThTlConfReg[0] &&
			(int8_t)ScratchpadData[3] == ThTlConfReg[1] &&
			ScratchpadData[4] == ThTlConfReg[2])
		return 0;
	else
		return 1;	// err
}

void ds18b20_GetThTlConfReg(const uint8_t * ScratchpadData, int8_t * ThTlConfReg){

	ThTlConfReg[0] = (int8_t)ScratchpadData[2];
	ThTlConfReg[1] = (int8_t)ScratchpadData[3];
	ThTlConfReg[2] = ScratchpadData[4];
}

void ds18b20_SetResolution(uint8_t resolution){
	mainResolution = resolution;

	if ( mainResolution == resolution_12bit )
		maxConversionTime = (uint16_t)999;
	else if ( mainResolution == resolution_11bit )
		maxConversionTime = (uint16_t)499;
	else if ( mainResolution == resolution_10bit )
		maxConversionTime = (uint16_t)249;
	else if ( mainResolution == resolution_9bit )
		maxConversionTime = (uint16_t)124;
}

uint8_t ds18b20_CopyScratchpad(const uint8_t * ROMcode){

	uint16_t presencePulseTime;

	presencePulseTime = ds18b20_initStep();
	if (presencePulseTime < 60 || presencePulseTime > 240){
		return 1;	//err
	}

	//  65 
		ds18b20_setTimerMode(0);
		TIM1->CNT = 0;
		TIM1->ARR = 65;

		inProcess = 1;
		NVIC_EnableIRQ(TIM1_UP_IRQn);
		//NVIC_EnableIRQ(TIM1_CC_IRQn);
		TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS; // ENABLE tim1;
		typeOp = 1;

		while ( inProcess );
		typeOp = 0;
	//  

	tempCommand = MatchRom;
	ds18b20_sendCommandData(&tempCommand, 0);
	ds18b20_sendCommandData(ROMcode, 64);

	tempCommand = CopyScratchpad;
	ds18b20_sendCommandData(&tempCommand, 0);

	//  10 
	ds18b20_setTimerMode(0);
	TIM1->CNT = 0;
	TIM1->ARR = 10500; // 10.5 

	inProcess = 1;
	NVIC_EnableIRQ(TIM1_UP_IRQn);
	TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS;     // ENABLE tim1;
	typeOp = 1;

	while ( inProcess );
	typeOp = 0;            //  
//  

	TIM1->ARR = 65; // ...

	return 0;
}

uint8_t ds18b20_RecallE2(const uint8_t * ROMcode){

	uint16_t presencePulseTime;

	presencePulseTime = ds18b20_initStep();
	if (presencePulseTime < 60 || presencePulseTime > 240){
		return 1;	//err
	}

	//  65 
		ds18b20_setTimerMode(0);
		TIM1->CNT = 0;
		TIM1->ARR = 65;

		inProcess = 1;
		NVIC_EnableIRQ(TIM1_UP_IRQn);
		TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS; // ENABLE tim1;
		typeOp = 1;

		while ( inProcess );
		typeOp = 0;            //  
	//  

	if ( ROMcode == 0 ){
		tempCommand = SkipRom;
		ds18b20_sendCommandData(&tempCommand, 0);
	}else{
		tempCommand = MatchRom;
		ds18b20_sendCommandData(&tempCommand, 0);
		ds18b20_sendCommandData(ROMcode, 64);
	}

	tempCommand = RecallE2;
	ds18b20_sendCommandData(&tempCommand, 0);

	ds18b20_getData(0, 0); //   

	return 0;
}

uint8_t ds18b20_ReadPowerSupply(uint8_t * userdataBuf){

	uint16_t presencePulseTime;

	presencePulseTime = ds18b20_initStep();
	if (presencePulseTime < 60 || presencePulseTime > 240){
		return 1;	//err
	}

	//  65 
		ds18b20_setTimerMode(0);
		TIM1->CNT = 0;
		TIM1->ARR = 65;

		inProcess = 1;
		NVIC_EnableIRQ(TIM1_UP_IRQn);
		TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS; // ENABLE tim1;
		typeOp = 1;

		while ( inProcess );
		typeOp = 0;
	//  

	tempCommand = SkipRom;
	ds18b20_sendCommandData(&tempCommand, 0);

	tempCommand = ReadPowerSupply;
	ds18b20_sendCommandData(&tempCommand, 0);
	//  userdataBuf   (  )
	ds18b20_getData(1, userdataBuf);

	return 0;
}


//  SearchRom :
//  1.     
//  2.     
//   ( 4      
//    Application Note 937 Book of iButton Standards)
uint8_t executeSearchAlarmRom(uint8_t Command, uint8_t (* parrayOfRomCodes)[8] , uint8_t * userdataBuf) {

	uint8_t rez, i = 0, y;
	struct SearchRomStr SearchRomStr1;

	SearchRomStr1.lastDiscrepancy = 0;
	SearchRomStr1.doneFlag = 0;

	do{
		rez = findNextROM(&SearchRomStr1, Command, userdataBuf);

		if (rez){ //  ROM  

			// rom-
			for ( y = 0; y < 8 ; y++ )
				parrayOfRomCodes[i][y] = SearchRomStr1.romBit[y];

			i++;
		}


	} while(rez);

	return i; //  i 
}

 //    :  0xA43C01D607E49328   {0x28, 0x93, 0xE4, 0x07, 0xD6, 0x01, 0x3C, 0xA4}

static uint8_t findNextROM(struct SearchRomStr * SearchRomStr1, uint8_t Command, uint8_t * userdataBuf){
	uint8_t romBitIndex = 1, LocalBitIndex = 0,discrepancyMarker = 0, bitA, bitB, nbyte = 0;
	uint8_t retv = 0;
	uint16_t presencePulseTime;

	if( SearchRomStr1->doneFlag ){
		SearchRomStr1->doneFlag = 0;
		return retv;
	}

	presencePulseTime = ds18b20_initStep();
	 // was(is, has) presence signal detected ?
	if (presencePulseTime < 60 || presencePulseTime > 240){
		SearchRomStr1->lastDiscrepancy = 0;
		return retv;
	}
	// romBitIndex  discrepancyMarker  

	tempCommand = Command;
	ds18b20_sendCommandData(&tempCommand, 0);

again:	ds18b20_getData(2, userdataBuf);
	bitA = dataBuf[0] & mask_bit0;
	bitB = (dataBuf[0] & mask_bit1) >> 1;

	if ( (bitA == bitB) & (bitA == 1) ){ //bitA = bitB = 1
		SearchRomStr1->lastDiscrepancy = 0;
		return retv;
	}

	if ( (bitA == bitB) & (bitA == 0) ){ //bitA = bitB = 0
		//      

		if( romBitIndex == SearchRomStr1->lastDiscrepancy ){
			bitA = 1;
			goto send1;
		}else{
			if ( romBitIndex > SearchRomStr1->lastDiscrepancy ){
				bitA = 0;
				discrepancyMarker = romBitIndex;
				goto send1;
			}else{
				//    Rom-   ( )
				bitA = ((SearchRomStr1->romBit)[nbyte] >> LocalBitIndex) & 0x01;
				if ( bitA == 0 ){
					discrepancyMarker = romBitIndex;
					goto send1;
				}else{
					goto send1;
				}
			}

		}

	}else {

send1:  (SearchRomStr1->romBit)[nbyte] &= ~( (uint8_t)1 << LocalBitIndex); //  
		(SearchRomStr1->romBit)[nbyte] |= (bitA << LocalBitIndex);         //  

		tempCommand = bitA;
		ds18b20_sendCommandData(&tempCommand, 1);

		if ( (romBitIndex % 8) == 0 ){
			nbyte++;
			LocalBitIndex = 0;
		}else
			LocalBitIndex++;

		romBitIndex++;

		if ( romBitIndex > 64 ){
			SearchRomStr1->lastDiscrepancy = discrepancyMarker;
			if( SearchRomStr1->lastDiscrepancy == 0 )
				SearchRomStr1->doneFlag = 1;
			retv = 1;
		}else{
			goto again;
		}

	}

	return retv;
}

uint8_t	CRCCalculation(const uint8_t * data, uint8_t nbit){
	uint8_t vCRC = 0, i, byte1, feedbackBit, byteNum = 0, newbit;

	byte1 = data[byteNum];

	for ( i = 1; i <= nbit; i++ ){

		feedbackBit = (byte1 & (uint8_t)1) ^ (vCRC & (uint8_t)1);
		vCRC >>= 1;

		vCRC = vCRC & 0x7F; //   
		if ( feedbackBit )
			vCRC = vCRC | 0x80; //   

		newbit = (vCRC & 0x08) ^ ( feedbackBit << 3 );
		vCRC = vCRC & 0xF7; //   3
		vCRC = vCRC | newbit;

		newbit = (vCRC & 0x04) ^ ( feedbackBit << 2 );
		vCRC = vCRC & 0xFB; //   2
		vCRC = vCRC | newbit;

		if ( i % 8 == 0 ){
			byte1 = data[++byteNum];
		}else
			byte1 = byte1 >> 1;
	}

	return vCRC;
}

uint8_t ds18b20_readROM(uint8_t * userdataBuf){
	uint16_t presencePulseTime;

	presencePulseTime = ds18b20_initStep();
	if (presencePulseTime < 60 || presencePulseTime > 240){
		return 1;	//err
	}

	//  65 
		ds18b20_setTimerMode(0);
		TIM1->CNT = 0;
		TIM1->ARR = 65;

		inProcess = 1;
		NVIC_EnableIRQ(TIM1_UP_IRQn);
		//NVIC_EnableIRQ(TIM1_CC_IRQn);
		TIM1->CR1 |= TIM_CR1_CEN | TIM_CR1_URS; // ENABLE tim1;
		typeOp = 1;

		while ( inProcess );
		typeOp = 0;
	//  

	tempCommand = ReadRom;
	ds18b20_sendCommandData(&tempCommand, 0);
	ds18b20_getData(64, userdataBuf);

	return 0;
}

////     /////  ///
/////////////////////////////////////////////////////////////////

/*
static uint8_t id1 = 0xAC; // 10101100
static uint8_t id2 = 0x55; // 01010101
static uint8_t id3 = 0xAF; // 10101111
static uint8_t id4 = 0x88; // 10001000

static uint8_t en1 = 1;
static uint8_t en2 = 1;
static uint8_t en3 = 1;
static uint8_t en4 = 1;

static uint8_t TEST_findNextROM(struct SearchRomStr * SearchRomStr1);

void TEST_executeSearchRom(void) {

	uint8_t rez, i;
	struct SearchRomStr SearchRomStr1;

	SearchRomStr1.lastDiscrepancy = 0;
	SearchRomStr1.doneFlag = 0;

	do{

		en1 = 1;
		en2 = 1;
		en3 = 1;
		en4 = 1;

		rez = TEST_findNextROM(&SearchRomStr1);

		if (rez){ //  ROM  
			// ,  ROM   "" 
		}


	} while(rez);

}
static uint8_t TEST_buff;

static void TEST_ds18b20_getData(uint8_t romBitIndex){
	uint8_t i = 0, bitA, bitB, bitAnext, bitBnext, y;
	uint8_t arr[4];

	//TEST_buff = 0;

	if ( en1 ){
		arr[i] = id1;
		i++;
	}

	if ( en2 ){
		arr[i] = id2;
		i++;
	}

	if ( en3 ){
		arr[i] = id3;
		i++;
	}

	if ( en4 ){
		arr[i] = id4;
		i++;
	}

	if ( i == 1 ){
		bitA = (arr[0] >> romBitIndex) & mask_bit0 ;
		if ( bitA ){
			bitB = 0;
		} else {
			bitB = 2; // 00000010
		}

	}else{
		bitA = (arr[0] >> romBitIndex) & mask_bit0 ;
		if ( bitA ){
			bitB = 0;
		} else {
			bitB = 2; // 00000010
		}

		for ( y = 1; y < i; y++ ){
			bitAnext = ( (arr[y] >> romBitIndex) & mask_bit0 );
			if ( bitAnext ){
				bitBnext = 0;
			} else {
				bitBnext = 2; // 00000010
			}

			bitA = bitA & bitAnext;
			bitB = bitB & bitBnext;
		}
	}


	TEST_buff = bitA | bitB;

}

//     
static void TEST_sendData(uint8_t bit, uint8_t romBitIndex){

	if ( en1 ){
		if ( ((id1 >> romBitIndex) & (uint8_t)1) != bit )
			en1 = 0;
	}

	if ( en2 ){
		if ( ((id2 >> romBitIndex) & (uint8_t)1) != bit )
			en2 = 0;
	}

	if ( en3 ){
		if ( ((id3 >> romBitIndex) & (uint8_t)1) != bit )
			en3 = 0;
	}

	if ( en4 ){
		if ( ((id4 >> romBitIndex) & (uint8_t)1) != bit )
			en4 = 0;
	}

}

static uint8_t TEST_findNextROM(struct SearchRomStr * SearchRomStr1){
	uint8_t romBitIndex = 1, LocalBitIndex = 0,discrepancyMarker = 0, bitA, bitB, nbyte = 0;
	uint8_t retv = 0;

	if( SearchRomStr1->doneFlag ){
		SearchRomStr1->doneFlag = 0;
		return retv;
	}

	// romBitIndex  discrepancyMarker  

again:	TEST_ds18b20_getData(romBitIndex-1); // 2 
	bitA = TEST_buff & mask_bit0;
	bitB = (TEST_buff & mask_bit1) >> 1;

	if ( (bitA == bitB) & (bitA == 1) ){ //bitA = bitB = 1
		SearchRomStr1->lastDiscrepancy = 0;
		return retv;
	}

	if ( (bitA == bitB) & (bitA == 0) ){ //bitA = bitB = 0
		//      

		if( romBitIndex == SearchRomStr1->lastDiscrepancy ){
			bitA = 1;
			goto send1;
		}else{
			if ( romBitIndex > SearchRomStr1->lastDiscrepancy ){
				bitA = 0;
				discrepancyMarker = romBitIndex;
				goto send1;
			}else{
				// !!!!!!     rom    ( )
				bitA = ((SearchRomStr1->romBit)[nbyte] >> LocalBitIndex) & 0x01;
				if ( bitA == 0 ){
					discrepancyMarker = romBitIndex;
					goto send1;
				}else{
					goto send1;
				}
			}

		}

	}else {

send1:  (SearchRomStr1->romBit)[nbyte] &= ~( (uint8_t)1 << LocalBitIndex); //  
		(SearchRomStr1->romBit)[nbyte] |= (bitA << LocalBitIndex);         //  

		TEST_sendData(bitA, romBitIndex-1);

		if ( (romBitIndex % 8) == 0 ){
			nbyte++;
			LocalBitIndex = 0;
		}else
			LocalBitIndex++;

		romBitIndex++;

		if ( romBitIndex > 8 ){
			SearchRomStr1->lastDiscrepancy = discrepancyMarker;
			if( SearchRomStr1->lastDiscrepancy == 0 )
				SearchRomStr1->doneFlag = 1;
			retv = 1;
		}else{
			goto again;
		}

	}

	return retv;
}
*/
/////////////////  /////////////////////////////
///////////////////////////////////////////////////////////////////
