/* ds18b20.h .  */

#include <stm32f10x.h>

#ifndef _DS18b20_H
#define _DS18b20_H

#define pin_main_signal GPIO_Pin_8  // gpioA.    .

// 
// (ROM commands)
//     64- (Rom-) 
// (      
//     ,     .)
#define SearchRom 0xF0 //    
#define ReadRom 0x33   // Rom-,    1 
#define MatchRom 0x55 //  
#define SkipRom 0xCC //" "
#define AlarmSearch 0xEC //     (  )

//   :
//(Scratchpad -      ,    )
#define ConvertT       0x44 //  
#define WriteScratchpad 0x4E //  (3 )  Scratchpad (.   )
#define ReadScratchpad 0xBE // Scratchpad (  ).
#define CopyScratchpad 0x48 //  (3 )  Scratchpad    (. . )
#define RecallE2       0xB8 //      Scratchpad
#define ReadPowerSupply 0xB4 //  :   ?

#define dataBufLen 9	//  .    

//    
#define mask_sign 0x800 //   11-  
#define mask_bit0 0x1
#define mask_bit1 0x2
#define mask_bit2 0x4
#define mask_bit3 0x8

//   
//        .
#define resolution_9bit  ( uint8_t )0x1F // 0b00011111  // :  93.75 
#define resolution_10bit ( uint8_t )0x3F // 0b00111111 //  187.5 
#define resolution_11bit ( uint8_t )0x5F // 0b01011111 //  375 
#define resolution_12bit ( uint8_t )0x7F // 0b01111111 //  750 

//      .
//    (Reset pulse)
//      (Presence pulse)  
//     ,   260      0.
uint16_t ds18b20_initStep(void);

//      Scratchpad
//        
//  1  4  (. ds18b20_WriteScratchpad() )
//
// pScratchpadData -     Scratchpad
// :  
float ds18b20_convertTemp(const uint8_t * pScratchpadData);

//       
// (1, ...)
void ds18b20_init(void);

//  SearchRom -    (ROM code)    
//  AlarmSearch -  ROM-     
//
//   Command: SearchRom  AlarmSearch
//  parrayOfRomCodes -     N   8  ( 64- Rom-)
//  userdataBuf -    (    )
//    userdataBuf = 1 
//  :   ,
//     rom-    arrayOfRomCodes
//
// !!!  Rom-   .  ""   .
// !!!   ,         
// !!!   (  ).
//
//   -   : Application Note 937 Book of iButton Standards
//    "MicroLAN - Networking Capabilities"
uint8_t executeSearchAlarmRom(uint8_t Command, uint8_t (* parrayOfRomCodes)[8] , uint8_t * userdataBuf);

//  CRC
//        ( )
//      ()  -  CRC (    
//   ReadScratchpad, ReadRom, SearchRom),     
//      (   CRC  CRC).
//       CRC   , 
//       CRC.
// ( "Understanding and Using Cyclic Redundancy Cheks with Maxim
//  1-Wire and iButton Prodects")
uint8_t	CRCCalculation(const uint8_t * data, uint8_t nbit);

//  64- Rom- .
// ,       ,     .
//
// userdataBuf -    Rom-
// : 0 -  , 1 - 
uint8_t ds18b20_readROM(uint8_t * userdataBuf);

//   ConvertT     ( ROMcode = 0)  .
//
// ROMcode -   64-  .
//   0,      (
//  MatchRom   SkipRom).
//
// : 0 -  , 1 - 
//
// !   ds18b20_ConvertT()    
// ""  
//        .
//     ""   , 
//     ,     
//  .        
//   ,    
//     .
//        
//   ds18b20_readScratchpad  ds18b20_GetThTlConfReg.
//     ConfReg.
uint8_t ds18b20_ConvertT(const uint8_t * ROMcode);

//   Scratchpad      ROMcode.
//
// ROMcode -   64-  .
//   0,  ,        
//  MatchRom   SkipRom.
//
//   userdataBuf  Scratchpad(9 )
// : 0 -  , 1 - 
uint8_t ds18b20_readScratchpad(const uint8_t * ROMcode, uint8_t * userdataBuf);

//       ThTlConfReg.
// ThTlConfReg -    :
//  Th( ),
//  Tl ( ) 
//  ConfReg( ).
//   ConfReg    resolution_9bit... 12bit.
//
//    ConvertT      
//  ""   Th()  Tl().
//      ,    
//  .         
//    ,   .
//           
//  AlarmSearch.
//
//     Th  Tl.
//     (ConfReg),
//      .
// ROMcode -   64-    
// ThTlConfReg -      : Th, Tl  ConfReg.
//
// : 0 -  , 1 - 
uint8_t ds18b20_WriteScratchpad(const uint8_t * ROMcode, const int8_t * ThTlConfReg);

// (  Th, Tl  ConfReg  ds18b20_WriteScratchpad()  
// Scratchpad     .)
//
//     Th, Tl  ConfReg  Scratchpad.
// ScratchpadData -    ,   Scratchpad
// ThTlConfReg -   ""  Th, Tl  ConfReg, 
//    ds18b20_WriteScratchpad()
//
// : 0 -  , 1 - 
uint8_t ds18b20_TestThTlConfReg(const uint8_t * ScratchpadData, const int8_t * ThTlConfReg);

// ScratchpadData -    ,   Scratchpad
// ( Th, Tl  ConfReg      2   )
// ThTlConfReg -   ,     Th, Tl  ConfReg
// (    ThTlConfReg   !)
void ds18b20_GetThTlConfReg(const uint8_t * ScratchpadData, int8_t * ThTlConfReg);

//     ( 9  12 )   .
// (      ds18b20_WriteScratchpad() )
//
//        Scratchpad  ds18b20_WriteScratchpad()
//  ( , ,     ds18b20_CopyScratchpad )
//        .
//     - 12 .       
//   750 .   9  -  8  .   
//  (ds18b20_WriteScratchpad())  9 ,       
//  (12),       (750   93.75 ).
//   ,       .
//
//     resolution_9bit..resolution_12bit
void ds18b20_SetResolution(uint8_t resolution);

//   CopyScratchpad    Th, Tl  ConfReg    .
//      10 (  )
//
// ROMcode -   64-    
//
// : 0 -  , 1 - 
uint8_t ds18b20_CopyScratchpad(const uint8_t * ROMcode);

//    Th, Tl  ConfigurationReg     Scratchpad.
// (           .)
//
//      .
// (     RecallE2      100 .)
//
// ROMcode -   64-    
//  ROMcode = 0,     .
//
// : 0 -  , 1 - 
uint8_t ds18b20_RecallE2(const uint8_t * ROMcode);

//      (  ) .
//
//  userdataBuf  ( 1 ):
//		0     
//      1     
//
// : 0 -  , 1 - 
uint8_t ds18b20_ReadPowerSupply(uint8_t * userdataBuf);

/*    ( 1 )
 *    desBitsPar.
 *  desBitsPar = 0,     ,     (8 )
 *    desBitsPar .*/
// (   ,  .)
void ds18b20_sendCommandData(const uint8_t * command, uint8_t desBitsPar);

/*  desBitsPar     .
     0 (),     
       ( RecallE2).
   ( :   "1".
      0,  userdataBuf ()   0.
*/
//  desBitsPar > 0,   userdataBuf   
//    (,   61 ,   8 ).
// (   ,  .)
//
void ds18b20_getData(uint8_t desBitsPar, uint8_t * userdataBuf);

//    
// (   )
void ds18b20_setTimerMode(uint8_t input);

//    .
void initDebug(void);    			// !!! 

//     
void TEST_executeSearchRom(void);	// !!! .


#endif // _DS18b20_H
