/* main.c */

#include <stm32f10x.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_gpio.h>
#include <stm32f10x_tim.h>
#include <misc.h>
#include "ds18b20.h"

#define pin_button GPIO_Pin_4  //   gpioA
#define maxDevNum 3  //     

void Delay(uint32_t nTime);
uint8_t debounce(uint8_t last);

//   ROM-
uint8_t arrayOfRomCodes[maxDevNum][8];
//    (    Scratchpad)
uint8_t dataBuf[dataBufLen];
//  Th, Tl  Configuration registor.
int8_t ThTlConfReg[3];   //  

int main(void)
{
	uint8_t currentButton;
	uint8_t lastButton;

	float temperature;
	uint8_t devNum, vCRC;

	GPIO_InitTypeDef	GPIO_InitStructure;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

	// Configure SysTick Timer
	/*(3)*/
	if (SysTick_Config(SystemCoreClock / 1000))
			while (1);

	//init Button's pin
	GPIO_InitStructure.GPIO_Pin = pin_button;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	ds18b20_init();

	//initDebug();   //   

    while(1)
    {
    	currentButton = debounce(lastButton);


    	if ( lastButton == Bit_RESET && currentButton == Bit_SET ){// 

    		//(0.)
    		//    .
			uint8_t buf1;
			if ( !ds18b20_ReadPowerSupply(&buf1) ){
				if( buf1 ){
					//    

				}else{
					//    
					//(!  )

				}
			}else{
				//err
			}

			//(1.)
			//     .
			//      ,    ( )
			// rom-     Th, Tl  ConfReg.
			devNum = executeSearchAlarmRom(SearchRom, arrayOfRomCodes, dataBuf);
			if ( devNum > 0 ){

				ThTlConfReg[0] = 25;	// Th
				ThTlConfReg[1] = -5;	// Tl
				ThTlConfReg[2] = resolution_9bit;	// ConfReg

				if( !ds18b20_WriteScratchpad(arrayOfRomCodes[0], ThTlConfReg)){

					if ( !ds18b20_readScratchpad(arrayOfRomCodes[0], dataBuf) ){
						//    (ScratchPad)
						vCRC = CRCCalculation(dataBuf, 72);
						if( vCRC == 0 ){ //   
							//    
							// Th, Tl  ConfReg(   )
							if ( !ds18b20_TestThTlConfReg(dataBuf, ThTlConfReg)){
								//     .
								//     
								//  .
								ds18b20_SetResolution(resolution_9bit);

								// ,     
								//   Th, Tl  ConfReg 
								//  ,  
								//   .
								if(!ds18b20_CopyScratchpad(arrayOfRomCodes[0])){
									// ok
								}
							}
						}
					}
				}
			}

			//(2.)
			//   ,   (1.),  . ,  
			//   .
			//
			devNum = executeSearchAlarmRom(SearchRom, arrayOfRomCodes, dataBuf);
			if ( devNum > 0 ){

				ThTlConfReg[0] = 25;	// Th
				ThTlConfReg[1] = -5;	// Tl
				ThTlConfReg[2] = resolution_9bit;	// ConfReg

				ds18b20_WriteScratchpad(arrayOfRomCodes[0], ThTlConfReg);
				ds18b20_SetResolution(resolution_9bit);
				//,  
				ds18b20_CopyScratchpad(arrayOfRomCodes[0]);

			}

    		//(3.)
    		//     .
    		//      ,    ( )
    		// Rom-    .
    		//
			// !     ""  
			// .   ds18b20_ConvertT().
			//
    		devNum = executeSearchAlarmRom(SearchRom, arrayOfRomCodes, dataBuf);
			if ( devNum > 0 ){
				//   
				if ( !ds18b20_ConvertT(arrayOfRomCodes[0]) ){
					if( !ds18b20_readScratchpad(arrayOfRomCodes[0], dataBuf)){
						//    (ScratchPad)
						vCRC = CRCCalculation(dataBuf, 72);
						if( vCRC == 0 ){ //   
							temperature = ds18b20_convertTemp(dataBuf);
						}
					}
				}
			}

			//(4.)
			//   ,   (3.),  . ,  
			//   .
			//
			devNum = executeSearchAlarmRom(SearchRom, arrayOfRomCodes, dataBuf);
			if ( devNum > 0 ){
				//   
				ds18b20_ConvertT(arrayOfRomCodes[0]);
				//    
				ds18b20_readScratchpad(arrayOfRomCodes[0], dataBuf);
				//       
				temperature = ds18b20_convertTemp(dataBuf);
			}


			//(5.)
			//      
			//
			devNum = executeSearchAlarmRom(AlarmSearch, arrayOfRomCodes, dataBuf);
			if ( devNum > 0 ){
				//ok
				//   ( )  
			}

			//(6.)
			//    " "
			//
			if(!ds18b20_ConvertT(0)){
				// ok
			}


			//(7.)
			// rom- . ,     .
			//
			//rom-     arrayOfRomCodes
			if(!ds18b20_readROM( arrayOfRomCodes[1] ) ){
				//ok
			}

			//(8.)
			//  Th, Tl  ConfReg   
			//  Scratchpad.
			//
			//   rom-
			if(!ds18b20_RecallE2( arrayOfRomCodes[1] )){
				//ok
			}


    	}

    	lastButton = currentButton;
    }
}

static __IO uint32_t TimingDelay;

void Delay(uint32_t nTime){
	TimingDelay = nTime;
	while(TimingDelay != 0);
}

void SysTick_Handler(void){
	if (TimingDelay != 0x00)
		TimingDelay--;
}

//    
uint8_t debounce(uint8_t last){
	uint8_t current = GPIO_ReadInputDataBit(GPIOA, pin_button);
	  if ( last != current ){
		  Delay(5);
		  current = GPIO_ReadInputDataBit(GPIOA, pin_button);
	  }
  return current;
}

#ifdef USE_FULL_ASSERT
void assert_failed(uint8_t *file, uint32_t line)
{
	/* Infinite loop */
	/* Use GDB to find out why we're here */
	while (1);
}
#endif
